@@ -13,8 +13,7 @@ import time
 
 from monasca_agent.collector.checks.services_checks import ServicesCheck
 from monasca_agent.collector.checks.services_checks import Status
-
-DETAIL_MAX_LEN = 2034  # The 2048 value_meta limit - the 14 characters used in the detail wrapper: len('{"detail": ""}')
+import monasca_agent.common.aggregator as aggregator
 
 
 class WrapNagios(ServicesCheck):
@@ -109,9 +108,13 @@ class WrapNagios(ServicesCheck):
 
         status_code = proc.poll()
         if detail:
+            value_meta = {'detail': detail}
+            overage = aggregator.get_value_meta_overage(value_meta)
+            if overage:
+                value_meta = {'detail': detail[:-overage]}
             self.gauge(metric_name, status_code,
                        dimensions=dimensions,
-                       value_meta={'detail': detail[0:DETAIL_MAX_LEN]})
+                       value_meta=value_meta)
         else:
             self.gauge(metric_name, status_code, dimensions=dimensions)
         # Return DOWN on critical, UP otherwise
