@@ -246,12 +246,14 @@ class Paths(object):
         raise PathNotFound(bad_path)
 
     def _unix_confd_path(self):
-        path = os.path.join(os.path.dirname(configuration.DEFAULT_CONFIG_FILE), 'conf.d')
-        path2 = os.path.join(os.getcwd(), 'conf.d')
+        try:
+            return configuration.Config().get_confd_path()
+        except PathNotFound:
+            pass
+
+        path = os.path.join(os.getcwd(), 'conf.d')
         if os.path.exists(path):
             return path
-        elif os.path.exists(path2):
-            return path2
         raise PathNotFound(path)
 
     def get_checksd_path(self):
@@ -444,6 +446,9 @@ def get_parsed_args():
     parser.add_option('-c', '--clean', action='store_true', default=False, dest='clean')
     parser.add_option('-v', '--verbose', action='store_true', default=False, dest='verbose',
                       help='Print out stacktraces for errors in checks')
+    parser.add_option('-f', '--config-file', default=None, dest='config_file',
+                      help='Location for an alternate config rather than '
+                           'using the default config location.')
 
     try:
         options, args = parser.parse_args()
